@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.75 1999/09/07 20:07:52 nisse Exp $ */
+ * $Id: client.c,v 1.76 1999/09/20 21:39:59 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -389,10 +389,16 @@ do_client_io(struct command *s UNUSED,
   channel->receive = do_receive;
   
   session->out->super.close_callback
-    = session->err->super.close_callback = make_channel_close(channel);
+    = session->err->super.close_callback
+    = make_channel_close_callback(channel);
   
   session->in->super.read = make_channel_read_data(channel);
 
+#if 0
+  session->in->super.close_callback
+    = make_channel_read_close_callback(channel);
+#endif
+  
   channel->send = do_send_first;
 
   ALIST_SET(channel->request_types, ATOM_EXIT_STATUS,
