@@ -1,6 +1,6 @@
 /* command.h
  *
- * $Id: command.h,v 1.35 1999/09/06 22:15:11 nisse Exp $ */
+ * $Id: command.h,v 1.36 1999/09/21 22:25:05 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -241,8 +241,8 @@ struct lsh_object *collect_trace(const char *name, struct lsh_object *real);
 #endif
 
 struct command_continuation *
-make_once_continution(const char *msg,
-		      struct command_continuation *up);
+make_once_continuation(const char *msg,
+		       struct command_continuation *up);
 
 /* extern struct command once_command; */
 
