@@ -1,6 +1,6 @@
 /* exception.h
  *
- * $Id: exception.h,v 1.9 1999/09/07 16:52:36 nisse Exp $ */
+ * $Id: exception.h,v 1.10 1999/09/20 21:43:35 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -75,9 +75,11 @@
 #define EXC_IO_OPEN_WRITE 0x2006
 #define EXC_IO_LISTEN 0x2007
 
+#if 0
 /* Not really errors */
 /* EOF was read */
 #define EXC_IO_EOF 0x2010
+#endif
 
 /* Authorization errors */
 #define EXC_AUTH 0x4000
