@@ -1,6 +1,6 @@
 /* io_commands.c
  *
- * $Id: io_commands.c,v 1.17 1999/08/22 21:24:12 nisse Exp $
+ * $Id: io_commands.c,v 1.20 1999/09/07 16:57:22 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -32,12 +32,95 @@
 
 #include <assert.h>
 
+/* Needed only to get the error code from failed calls to io_connect() */
+#include <errno.h>
+
+/* For STDIN_FILENO */
+#include <unistd.h>
+
 #define GABA_DEFINE
 #include "io_commands.h.x"
 #undef GABA_DEFINE
 
 #include "io_commands.c.x"
 
+/* GABA:
+   (class
+     (name backend_command)
+     (super command)
+     (vars
+       (backend object io_backend)))
+*/
+
+/* (write file_info backend)
+ *
+ * Opens a file for write, and returns the corresponding write_buffer.
+ * */
+   
+static void
+do_io_write_file(struct command *s,
+		 struct lsh_object *a,
+		 struct command_continuation *c,
+		 struct exception_handler *e)
+{
+  CAST(backend_command, self, s);
+  CAST(io_write_file_info, info, a);
+
+  struct io_fd *fd = io_write_file(self->backend,
+				   info->name,
+				   info->flags,
+				   info->mode,
+				   info->block_size,
+				   NULL,
+				   e);
+  if (fd)
+    COMMAND_RETURN(c, fd->write_buffer);
+  else
+    EXCEPTION_RAISE(e, make_io_exception(EXC_IO_OPEN_WRITE, NULL, errno, NULL));
+}
+
+static struct lsh_object *
+do_io_write_file_collect(struct command_simple *s UNUSED,
+			 struct lsh_object *a)
+{
+  CAST(io_backend, backend, a);
+
+  NEW(backend_command, self);
+  self->super.call = do_io_write_file;
+  self->backend = backend;
+
+  return &self->super.super;
+}
+
+struct command_simple io_write_file_command
+= STATIC_COMMAND_SIMPLE(do_io_write_file_collect);
+
+struct io_write_file_info *
+make_io_write_file_info(const char *name, int flags, int mode, UINT32 block_size)
+{
+  NEW(io_write_file_info, self);
+  self->name = name;
+  self->flags = flags;
+  self->mode = mode;
+  self->block_size = block_size;
+
+  return self;
+}
+
+void do_io_read_fd(struct command *s,
+		   struct lsh_object *a,
+		   struct command_continuation *c,
+		   struct exception_handler *e)
+{
+  CAST(io_read_fd, self, s);
+  CAST(io_backend, backend, a);
+
+  COMMAND_RETURN(c, make_io_fd(backend, self->fd, e));
+}
+
+struct io_read_fd io_read_stdin
+= STATIC_IO_READ_FD(STDIN_FILENO);
+
 /* GABA:
    (class
      (name listen_command_callback)
@@ -98,9 +181,9 @@ do_listen(struct io_backend *backend,
 
   if (!fd)
     {
-      /* FIXME: Raise a real exception. Possibly including errno? */
       /* NOTE: Will never invoke the continuation. */
-      EXCEPTION_RAISE(e, &dummy_exception);
+      EXCEPTION_RAISE(e, make_io_exception(EXC_IO_LISTEN,
+					   NULL, errno, NULL));
       return;
     }
   
@@ -181,7 +264,7 @@ struct collect_info_1 listen_command =
 STATIC_COLLECT_1(&collect_info_listen_2);
 
 
-
+/* FIXME: This could perhaps be merged with io_connect in io.c? */ 
 /* GABA:
    (class
      (name connect_command_callback)
@@ -192,11 +275,6 @@ STATIC_COLLECT_1(&collect_info_listen_2);
        (e object exception_handler)))
 */
 
-#if 0
-static struct exception connect_exception =
-STATIC_EXCEPTION(EXC_CONNECT, "connect failed");
-#endif
-
 static void
 do_connect_continue(struct fd_callback **s, int fd)
 {
@@ -204,14 +282,8 @@ do_connect_continue(struct fd_callback **s, int fd)
 
   assert(fd >= 0);
 
-#if 0
-  if (fd < 0)
-    return EXCEPTION_RAISE(self->e, &connect_exception);
-#endif
-
   /* FIXME: What exception handler to use? */
-  COMMAND_RETURN(self->c, make_io_fd(self->backend, fd,
-				     &default_exception_handler));
+  COMMAND_RETURN(self->c, make_io_fd(self->backend, fd, self->e));
 }
 
 static struct fd_callback *
@@ -249,12 +321,12 @@ do_connect(struct io_backend *backend,
     }
 
   fd = io_connect(backend, &sin, NULL,
-		  make_connect_command_callback(backend, c, e));
+		  make_connect_command_callback(backend, c, e),
+		  e);
 
-  /* FIXME: Use a proper io-exception */
   if (!fd)
     {
-      EXCEPTION_RAISE(e, &dummy_exception);
+      EXCEPTION_RAISE(e, make_io_exception(EXC_IO_CONNECT, NULL, errno, NULL));
       return;
     }
 
