@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.71 1999/09/07 20:07:57 nisse Exp $ */
+ * $Id: lsh.c,v 1.73 1999/09/21 22:50:19 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -232,9 +232,11 @@ do_lsh_default_handler(struct exception_handler *s,
 
       switch(e->type)
 	{
+#if 0
 	case EXC_IO_EOF:
 	  close_fd_nicely(exc->fd, 0);
 	  break;
+#endif
 	default:
 	  *self->status = EXIT_FAILURE;
 	  EXCEPTION_RAISE(self->super.parent, e);
@@ -365,7 +367,7 @@ int main(int argc, char **argv)
 	{ "forward-remote-port", required_argument, NULL, 'R'},
 #endif /* WITH_TCP_FORWARD */
 	{ "nop", no_argument, NULL, 'N' },
-	{ NULL }
+	{ NULL, 0, NULL, 0 }
       };
       
       option = getopt_long(argc, argv, "+c:l:np:qgtvz:L:R:N", options, NULL);
