@@ -1,6 +1,6 @@
 /* tcpforward_commands.c
  *
- * $Id: tcpforward_commands.c,v 1.12 1999/09/07 20:08:10 nisse Exp $
+ * $Id: tcpforward_commands.c,v 1.13 1999/09/20 22:37:36 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -403,12 +403,12 @@ make_forward_local_port(struct io_backend *backend,
      (expr
        (lambda (connection)
          (remote_listen (lambda (peer)
-	                  (connect_io ; peer
-			            ;; FIXME: This calls connect too early
-				    ;; Let it take peer as argument
-				    ;; Replacing connection with (K connection peer)
-				    ;; might work, except that it is optimized away.
-			            (connect connection)))
+	                  (connect_io 
+			     ; NOTE: The use of prog1 is needed to
+			     ; delay the connect call until the
+			     ; (otherwise ignored) peer argument is
+			     ; available.  
+			     (connect (prog1 connection peer))))
 	                remote
 			connection))))
 */
